/**
 * Copyright (C) 2010 the original author or authors.
 * See the notice.md file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.beust.jcommander.args;

import com.beust.jcommander.HostPort;
import com.beust.jcommander.HostPortConverter;
import com.beust.jcommander.Parameter;

import org.testng.collections.Lists;

import java.util.List;

/**
 * A class with main parameter that is not a List<String>
 * 
 * @author cbeust
 */
public class ArgsMainParameter2 implements IHostPorts {
  @Parameter(converter = HostPortConverter.class)
  public List<HostPort> parameters = Lists.newArrayList();

  public List<HostPort> getHostPorts() {
    return parameters;
  }
}
